/******************************************************************************
*
*    NTU  GIEE   MSOC   Final Project   2018/05
*
*
******************************************************************************/

/***************************** Include Files *********************************/
#include "xtime_l.h"
#include "xparameters.h"	/* SDK generated parameters */
#include "xsdps.h"		/* SD device driver */
#include "xil_printf.h"
#include "ff.h"
#include "xil_cache.h"
#include "xplatform_info.h"


/************************** Constant Definitions *****************************/
int FfsSdPolledRead(void);
int FfsSdPolledOpen(void);
int FfsSdPolledClose(void);
int FfsSdPolledWrite(void);
u8 mf(u8*);

/************************** Variable Definitions *****************************/
static FIL fil;		/* File object */
static FATFS fatfs;
static char FileName[32] = "STEFAN.Y";
static char *SD_File;
u32 Platform;

#ifdef __ICCARM__
#pragma data_alignment = 32
u8 InputFrame [352*288];
u8 OutputFrame [352*288*100];
#pragma data_alignment = 4
#else
u8 InputFrame [352*288] __attribute__ ((aligned(32)));
u8 OutputFrame [352*288*100] __attribute__ ((aligned(32)));
#endif






/*****************************************************************************/
/**
*
* Main function to call the SD example.
*
* @param	None
*
* @return	XST_SUCCESS if successful, otherwise XST_FAILURE.
*
* @note		None
*
******************************************************************************/



int main(void)
{
	//  NOTE! The cache should not be enable!!!
	Xil_DCacheDisable();

	int Status, f, i, j, idy, idx, index_y, index_x;
	u8  blk[9];
	u8* bp;
	u8* fp=OutputFrame;
	XTime tCur,tEnd;
	XTime_GetTime(&tCur);



	xil_printf("SD Polled File System Read \r\n");

	Status = FfsSdPolledOpen();
	if (Status != XST_SUCCESS) {
		xil_printf("SD Polled File System Open failed \r\n");
		return XST_FAILURE;
	}

	xil_printf("Successfully ran SD Polled File System Open \r\n");



	for(f=0;f<100;++f){
		//xil_printf("Now Processing Frame%d...\r\n",f);

		Status=FfsSdPolledRead();
		if (Status != XST_SUCCESS) {
			xil_printf("SD Polled File System Read failed \r\n");
			return XST_FAILURE;
		}
		for(i=0;i<288;++i)
			for(j=0;j<352;++j){

				bp=blk;

				for(idy=-1;idy<=1;++idy)
					for(idx=-1;idx<=1;++idx){
						index_y=(i+idy <0)?0:(i+idy>287)?287:i+idy;
						index_x=(j+idx <0)?0:(j+idx>351)?351:j+idx;
						*bp++=InputFrame[index_y*352+index_x];
					}



				*fp++=mf(blk);

		}

	}


	XTime_GetTime(&tEnd);
	float FPS = 10000.0/(tEnd-tCur)*COUNTS_PER_SECOND;
	xil_printf("FPS = %d/100 \r\n",(int)FPS);



	FfsSdPolledClose();
	if (Status != XST_SUCCESS) {
			xil_printf("SD Polled File System Close failed \r\n");
			return XST_FAILURE;
		}
	Status=FfsSdPolledWrite();
	if(Status!= XST_SUCCESS){
		xil_printf("Successfully ran SD Polled File System failed \r\n");
		return XST_FAILURE;
	}
	xil_printf("Successfully ran SD Polled File System Write \r\n");
	return XST_SUCCESS;

}

/*****************************************************************************/
/**
*
* File system example using SD driver to write to and read from an SD card
* in polled mode. This example creates a new file on an
* SD card (which is previously formatted with FATFS), write data to the file
* and reads the same data back to verify.
*
* @param	None
*
* @return	XST_SUCCESS if successful, otherwise XST_FAILURE.
*
* @note		None
*
******************************************************************************/

int FfsSdPolledWrite(void)
{
	FRESULT Res;
	UINT NumBytesRead;
	u32 FileSize = 352*288*100;
	//TCHAR *Path = "0:/";

	Platform = XGetPlatform_Info();


	/*
	 * Register volume work area, initialize device
	 */
	//Res = f_mount(&fatfs, Path, 0);

	/*if (Res != FR_OK) {
		return XST_FAILURE;
	}*/

	/*
	 * Open file with required permissions.
	 * Here - Creating new file with read/write permissions. .
	 * To open file with write permissions, file system should not
	 * be in Read Only mode.
	 */



	Res = f_open(&fil, "DENOISE.Y",  FA_CREATE_ALWAYS | FA_WRITE);
	if (Res) {
		return XST_FAILURE;
	}

	Res = f_lseek(&fil, 0);
		if (Res) {
			return XST_FAILURE;
		}

	Res = f_write(&fil, (void*)OutputFrame, FileSize,
				&NumBytesRead);
		if (Res) {
			return XST_FAILURE;
		}

	/*
	 * Pointer to beginning of file .
	 */
	Res=f_close(&fil);
	if (Res) {
			return XST_FAILURE;
		}

	return XST_SUCCESS;
}


int FfsSdPolledOpen(void){
	FRESULT Res;

	TCHAR *Path = "0:/";

	Platform = XGetPlatform_Info();


	/*
	 * Register volume work area, initialize device
	 */
	Res = f_mount(&fatfs, Path, 0);

	if (Res != FR_OK) {
		return XST_FAILURE;
	}

	/*
	 * Open file with required permissions.
	 * Here - Creating new file with read/write permissions. .
	 * To open file with write permissions, file system should not
	 * be in Read Only mode.
	 */

	SD_File = (char *)FileName;

	Res = f_open(&fil, SD_File,  FA_OPEN_EXISTING|FA_READ);
	if (Res) {
		return XST_FAILURE;
	}

	Res = f_lseek(&fil, 0);
	if (Res) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

int FfsSdPolledClose(void){
	FRESULT Res;
	Res=f_close(&fil);
		if (Res) {
				return XST_FAILURE;
			}
	return XST_SUCCESS;

}


int FfsSdPolledRead(void)
{
	FRESULT Res;
	UINT NumBytesRead;
	u32 FileSize = 352*288;




	Res = f_read(&fil, (void*)InputFrame, FileSize,
				&NumBytesRead);
		if (Res) {
			return XST_FAILURE;
		}

	/*
	 * Pointer to beginning of file .
	 */


	return XST_SUCCESS;
}


u8 mf(u8* blk){
	// 3x3 median filter by bubble sort
	int i, j , Temp,sp;
    for (i=8; i>0; i--)
       {
        sp=1;
       for (j =0; j <i; j++)
          if (blk[j] > blk[j+1])
             {
               Temp = blk[j];
               blk[j] = blk[j+1];
               blk[j+1] = Temp;
               sp=0;
             }
             if (sp==1) break;
       }
	return blk[4];
};

